@@ -2,18 +2,27 @@ import contextlib
 import json
 from copy import deepcopy
 from datetime import timedelta
-from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple, cast
 
 from psycopg2.sql import SQL, Identifier
-from splitgraph.core.types import Credentials, MountError, Params, TableInfo
+
+from splitgraph.core.types import (
+    Credentials,
+    MountError,
+    Params,
+    TableInfo,
+    TableParams,
+    TableSchema,
+)
 from splitgraph.hooks.data_source.fdw import (
     ForeignDataWrapperDataSource,
     import_foreign_schema,
 )
 from splitgraph.ingestion.common import IngestionAdapter, build_commandline_help
+from splitgraph.ingestion.csv.common import dump_options, load_options
 
 if TYPE_CHECKING:
-    from splitgraph.engine.postgres.engine import PsycopgEngine
+    from splitgraph.engine.postgres.engine import PostgresEngine, PsycopgEngine
 
 
 class CSVIngestionAdapter(IngestionAdapter):
@@ -25,7 +34,7 @@ class CSVIngestionAdapter(IngestionAdapter):
 
     @staticmethod
     def data_to_new_table(
-        data, engine: "PsycopgEngine", schema: str, table: str, no_header: bool = True, **kwargs
+        data, engine: "PostgresEngine", schema: str, table: str, no_header: bool = True, **kwargs
     ):
         copy_csv_buffer(data, engine, schema, table, no_header, **kwargs)
 
@@ -73,7 +82,10 @@ def query_to_csv(engine: "PsycopgEngine", query, buffer, schema: Optional[str] =
 class CSVDataSource(ForeignDataWrapperDataSource):
     credentials_schema: Dict[str, Any] = {
         "type": "object",
-        "properties": {"s3_access_key": {"type": "string"}, "s3_secret_key": {"type": "string"}},
+        "properties": {
+            "s3_access_key": {"type": "string", "title": "AWS Access Key"},
+            "s3_secret_key": {"type": "string", "title": "AWS Secret Access Key"},
+        },
     }
 
     params_schema: Dict[str, Any] = {
@@ -81,41 +93,60 @@ class CSVDataSource(ForeignDataWrapperDataSource):
         "properties": {
             "connection": {
                 "type": "object",
+                "title": "Connection",
                 "oneOf": [
                     {
                         "type": "object",
                         "required": ["connection_type", "url"],
                         "properties": {
-                            "connection_type": {"type": "string", "const": "http"},
-                            "url": {"type": "string", "description": "HTTP URL to the CSV file"},
+                            "connection_type": {
+                                "type": "string",
+                                "const": "http",
+                                "title": "Connection type",
+                            },
+                            "url": {
+                                "type": "string",
+                                "description": "HTTP URL to the CSV file",
+                                "title": "URL",
+                            },
                         },
                     },
                     {
                         "type": "object",
                         "required": ["connection_type", "s3_endpoint", "s3_bucket"],
                         "properties": {
-                            "connection_type": {"type": "string", "const": "s3"},
+                            "connection_type": {
+                                "type": "string",
+                                "const": "s3",
+                                "title": "Connection type",
+                            },
                             "s3_endpoint": {
                                 "type": "string",
+                                "title": "S3 endpoint",
                                 "description": "S3 endpoint (including port if required)",
                             },
                             "s3_region": {
                                 "type": "string",
+                                "title": "S3 region",
                                 "description": "Region of the S3 bucket",
                             },
                             "s3_secure": {
+                                "title": "Secure",
                                 "type": "boolean",
                                 "description": "Whether to use HTTPS for S3 access",
                             },
                             "s3_bucket": {
+                                "title": "Bucket name",
                                 "type": "string",
                                 "description": "Bucket the object is in",
                             },
                             "s3_object": {
+                                "title": "S3 Object name",
                                 "type": "string",
                                 "description": "Limit the import to a single object",
                             },
                             "s3_object_prefix": {
+                                "title": "S3 Object prefix",
                                 "type": "string",
                                 "description": "Prefix for object in S3 bucket",
                             },
@@ -125,36 +156,43 @@ class CSVDataSource(ForeignDataWrapperDataSource):
             },
             "autodetect_header": {
                 "type": "boolean",
+                "title": "Autodetect header",
                 "description": "Detect whether the CSV file has a header automatically",
                 "default": True,
             },
             "autodetect_dialect": {
                 "type": "boolean",
+                "title": "Autodetect dialect",
                 "description": "Detect the CSV file's dialect (separator, quoting characters etc) automatically",
                 "default": True,
             },
             "autodetect_encoding": {
                 "type": "boolean",
+                "title": "Autodetect encoding",
                 "description": "Detect the CSV file's encoding automatically",
                 "default": True,
             },
             "autodetect_sample_size": {
                 "type": "integer",
+                "title": "Sample size",
                 "description": "Sample size, in bytes, for encoding/dialect/header detection",
                 "default": 65536,
             },
             "schema_inference_rows": {
                 "type": "integer",
+                "title": "Schema inference rows",
                 "description": "Number of rows to use for schema inference",
                 "default": 100000,
             },
             "encoding": {
                 "type": "string",
+                "title": "Encoding",
                 "description": "Encoding of the CSV file",
                 "default": "utf-8",
             },
             "ignore_decode_errors": {
                 "type": "boolean",
+                "title": "Ignore decoding errors",
                 "description": "Ignore errors when decoding the file",
                 "default": False,
             },
@@ -164,11 +202,13 @@ class CSVDataSource(ForeignDataWrapperDataSource):
                 "default": True,
             },
             "delimiter": {
+                "title": "Delimiter",
                 "type": "string",
                 "description": "Character used to separate fields in the file",
                 "default": ",",
             },
             "quotechar": {
+                "title": "Quote character",
                 "type": "string",
                 "description": "Character used to quote fields",
                 "default": '"',
@@ -179,8 +219,12 @@ class CSVDataSource(ForeignDataWrapperDataSource):
     table_params_schema: Dict[str, Any] = {
         "type": "object",
         "properties": {
-            "url": {"type": "string", "description": "HTTP URL to the CSV file"},
-            "s3_object": {"type": "string", "description": "S3 object of the CSV file"},
+            "url": {"type": "string", "description": "HTTP URL to the CSV file", "title": "URL"},
+            "s3_object": {
+                "type": "string",
+                "description": "S3 object of the CSV file",
+                "title": "S3 object",
+            },
             # Add various CSV dialect overrides to the table params schema.
             **{k: v for k, v in params_schema["properties"].items() if k != "connection"},
         },
@@ -190,6 +234,8 @@ class CSVDataSource(ForeignDataWrapperDataSource):
     supports_load = True
     supports_sync = False
 
+    _icon_file = "csv.svg"
+
     commandline_help = """Mount CSV files in S3/HTTP.
 
 If passed an URL, this will live query a CSV file on an HTTP server. If passed
@@ -221,7 +267,7 @@ EOF
 
     def __init__(
         self,
-        engine: "PsycopgEngine",
+        engine: "PostgresEngine",
         credentials: Credentials,
         params: Params,
         tables: Optional[TableInfo] = None,
@@ -279,14 +325,24 @@ EOF
     def get_table_options(
         self, table_name: str, tables: Optional[TableInfo] = None
     ) -> Dict[str, str]:
-        result = super().get_table_options(table_name, tables)
+        tables = tables or self.tables
+
+        if not isinstance(tables, dict):
+            return {}
+
+        result = {
+            k: v
+            for k, v in tables.get(table_name, cast(Tuple[TableSchema, TableParams], ({}, {})))[
+                1
+            ].items()
+        }
 
         # Set a default s3_object if we're using S3 and not HTTP
         if "url" not in result:
             result["s3_object"] = result.get(
                 "s3_object", self.params.get("s3_object_prefix", "") + table_name
             )
-        return result
+        return dump_options(result)
 
     def _create_foreign_tables(
         self, schema: str, server_id: str, tables: TableInfo
@@ -334,29 +390,25 @@ EOF
 
         # Deserialize things like booleans from the foreign table options that the FDW inferred
         # for us.
-        def _destring_table_options(table_options):
-            for k in ["autodetect_header", "autodetect_encoding", "autodetect_dialect", "header"]:
-                if k in table_options:
-                    table_options[k] = table_options[k].lower() == "true"
-            return table_options
-
-        return [(t, _destring_table_options(d)) for t, d in options]
+        return [(t, load_options(d)) for t, d in options]
 
     def get_server_options(self):
-        options: Dict[str, Optional[str]] = {
-            "wrapper": "splitgraph.ingestion.csv.fdw.CSVForeignDataWrapper"
-        }
+        options: Dict[str, Any] = {}
         for k in self.params_schema["properties"].keys():
             # Flatten the options and extract connection parameters
             if k in self.params:
                 if k != "connection":
-                    options[k] = str(self.params[k])
+                    options[k] = self.params[k]
                 else:
                     options.update(self.params[k])
 
         for k in self.credentials_schema["properties"].keys():
             if k in self.credentials:
-                options[k] = str(self.credentials[k])
+                options[k] = self.credentials[k]
+        # Serialize all options to JSON so that we can be sure they're all strings.
+        options = dump_options(options)
+        options["wrapper"] = "splitgraph.ingestion.csv.fdw.CSVForeignDataWrapper"
+
         return options
 
     def get_remote_schema_name(self) -> str:
@@ -399,8 +451,9 @@ EOF
         # for each table.
         server_options = self.get_server_options()
         for table in tables:
-            table_options = super().get_table_options(table, tables)
+            table_options = self.get_table_options(table, tables)
             full_options = {**server_options, **table_options}
-            result[table] = self._get_url(full_options)
+            del full_options["wrapper"]
+            result[table] = self._get_url(load_options(full_options))
 
         return result
